@@ -40,9 +40,11 @@ from rospkg import RosPack, RosStack, get_ros_home, ResourceNotFound
 from rospkg.os_detect import OsDetect
 
 from .core import RosdepInternalError, rd_debug
-from .model import RosdepDatabase, RosdepDatabaseEntry, InvalidRosdepData
+from .model import RosdepDatabase, RosdepDatabaseEntry, InvalidData
 from .rospkg_loader import RosPkgLoader
 
+from .sources_list import create_default_matcher, SourcesListLoader
+
 # key for representing .ros/rosdep.yaml override entry
 OVERRIDE_ENTRY = '.ros'
 
@@ -77,13 +79,13 @@ class RosdepDefinition(object):
         :returns: (installer_key, rosdep_args_dict), ``(str, dict)``
 
         :raises: :exc:`ResolutionError` If no rule is available
-        :raises: :exc:`InvalidRosdepData` If rule data is not valid
+        :raises: :exc:`InvalidData` If rule data is not valid
         """
         rosdep_key = self.rosdep_key
         data = self.data
 
         if type(data) != dict:
-            raise InvalidRosdepData("rosdep value for [%s] must be a dictionary"%(self.rosdep_key), origin=self.origin)
+            raise InvalidData("rosdep value for [%s] must be a dictionary"%(self.rosdep_key), origin=self.origin)
         if os_name not in data:
             raise ResolutionError(rosdep_key, data, os_name, os_version, "No definition of [%s] for OS [%s]"%(rosdep_key, os_name))
         data = data[os_name]
@@ -110,7 +112,7 @@ class RosdepDefinition(object):
                     # version key must be present here for data to be valid
                     # dictionary value.
                     if os_version not in data:
-                        raise ResolutionError(rosdep_key, self.data, os_name, os_version, "No definition for OS version [%s]"%(os_version))
+                        raise ResolutionError(rosdep_key, self.data, os_name, os_version, "No definition of [%s] for OS version [%s]"%(rosdep_key, os_version))
                     data = data[os_version]
                     if type(data) == dict:
                         for installer_key in installer_keys:
@@ -120,7 +122,7 @@ class RosdepDefinition(object):
                                 break
 
         if type(data) not in (dict, list, type('str')):
-            raise InvalidRosdepData("rosdep OS definition for [%s:%s] must be a dictionary, string, or list: %s"%(self.rosdep_key, os_name, data), origin=self.origin)
+            raise InvalidData("rosdep OS definition for [%s:%s] must be a dictionary, string, or list: %s"%(self.rosdep_key, os_name, data), origin=self.origin)
 
         return return_key, data
 
@@ -147,33 +149,6 @@ class ResolutionError(Exception):
 \tOS version : %s
 \tData: %s"""%(self.args[0], self.rosdep_key, self.os_name, self.os_version, pretty_data)
 
-class RosdepConflict(Exception):
-    """
-    Rosdep rules do not have compatbile definitions.
-    """
-
-    def __init__(self, definition_name, definition1, definition2):
-        """
-        :apram definition_name: name of rosdep key that has conflicting definition.
-        :param definition1: Existing rule, :class:`RosdepDefinition`
-        :param definition2: Conflicting rule, :class:`RosdepDefinition`
-        """
-        self.definition_name = definition_name
-        self.definition1 = definition1
-        self.definition2 = definition2
-        
-    def __str__(self):
-        pretty_data1 = yaml.dump(self.definition1.data, default_flow_style=False)
-        pretty_data2 = yaml.dump(self.definition2.data, default_flow_style=False)
-        return """Rules for %s do not match:
-In [%s]
-
-%s 
-
-In [%s]
-
-%s"""%(self.definition_name, self.definition1.origin, pretty_data1, self.definition2.origin, pretty_data2)
-    
 class RosdepView(object):
     """
     View of :class:`RosdepDatabase`.  Unlike :class:`RosdepDatabase`,
@@ -202,32 +177,27 @@ class RosdepView(object):
         """
         return self.rosdep_defs.keys()
         
-    def merge(self, update_entry, override=False):
+    def merge(self, update_entry, override=False, verbose=False):
         """
-        Merge rosdep database update into main database
+        Merge rosdep database update into main database.  Merge rules
+        are first entry to declare a key wins.  There are no
+        conflicts.  This rule logic is modelled after the apt sources
+        list.
 
-        :raises: :exc:`RosdepConflict`
+        :param override: Ignore first-one-wins rules and instead
+            always use rules from update_entry
         """
         db = self.rosdep_defs
 
         for dep_name, dep_data in update_entry.rosdep_data.items():
             # convert data into RosdepDefinition model
             update_definition = RosdepDefinition(dep_name, dep_data, update_entry.origin)
+            # First rule wins or override, no rule-merging.
             if override or not dep_name in db:
                 db[dep_name] = update_definition
-            else:
-                definition = db[dep_name]
-                curr_data = definition.data
-                # First, check for conflict.  Conflict's are
-                # OS-specific.  We cannot check at a finer granularity
-                # as keys further in the hierarchy are opaque.
-                for k, v in dep_data.items():
-                    if k in curr_data and curr_data[k] != v:
-                        raise RosdepConflict(dep_name, definition, update_definition) 
-                
-                # If no conflict, do an update
-                curr_data.update(dep_data)
-
+            elif verbose:
+                print("[%s] ignoring [%s], already loaded"%(update_entry.origin, dep_name))
+            
 class RosdepLookup(object):
     """
     Lookup rosdep definitions.  Provides API for most
@@ -238,8 +208,7 @@ class RosdepLookup(object):
     has already been loaded.
     """
     
-    def __init__(self, rosdep_db, loader,
-                 override_entry=None):
+    def __init__(self, rosdep_db, loader, override_entry=None):
         """
         :param loader: Loader to use for loading rosdep data by stack
           name, ``RosdepLoader``
@@ -249,7 +218,7 @@ class RosdepLookup(object):
         """
         self.rosdep_db = rosdep_db
         self.loader = loader
-
+        
         self._view_cache = {} # {str: {RosdepView}}
         self._resolve_cache = {} # {str : (os_name, os_version, installer_key, resolution, dependencies)}
         
@@ -298,7 +267,9 @@ class RosdepLookup(object):
         return [k for k in self.loader.get_loadable_resources() if rosdep_name in self.get_rosdeps(k, implicit=False)]
 
     @staticmethod
-    def create_from_rospkg(rospack=None, rosstack=None, ros_home=None):
+    def create_from_rospkg(rospack=None, rosstack=None, ros_home=None,
+                           sources_matcher=None, use_underlay=True,
+                           verbose=False):
         """
         Create :class:`RosdepLookup` based on current ROS package
         environment.
@@ -307,6 +278,11 @@ class RosdepLookup(object):
           instance used to crawl ROS packages.
         :param rosstack: (optional) Override :class:`rospkg.RosStack`
           instance used to crawl ROS stacks.
+        :param ros_home: (optional) Override ROS_HOME.
+        :param use_underlay: (optional) Disable the source.list underlay
+        :param sources_matcher: (optional) Override matcher used to
+            determine which sources.list entries are
+            loadable. :class:`DataSourceMatcher`
         """
         # initialize the loader
         if rospack is None:
@@ -315,9 +291,31 @@ class RosdepLookup(object):
             rosstack = RosStack()
         if ros_home is None:
             ros_home = get_ros_home()
+        if sources_matcher is None:
+            sources_matcher = create_default_matcher()
 
-        loader = RosPkgLoader(rospack=rospack, rosstack=rosstack)
         rosdep_db = RosdepDatabase()
+
+        if use_underlay:
+            # Use underlay to initialize rosdep_db.  Underlay has no
+            # notion of specific resources, and its view keys are just the
+            # individual sources it can load from.  Unlike RosPkgLoader,
+            # SourcesListLoader cannot do delayed evaluation of OS setting
+            # due to matcher.
+            if verbose:
+                print("using matcher with tags [%s]"%(', '.join(sources_matcher.tags)))
+            underlay_loader = SourcesListLoader.create_default(sources_matcher)
+            underlay_key = SourcesListLoader.ALL_VIEW_KEY
+        else:
+            underlay_key = underlay_loader = None
+            
+        # Create the rospkg loader on top of the underlay
+        loader = RosPkgLoader(rospack=rospack, rosstack=rosstack,
+                              underlay_key=underlay_key)
+
+        # TODO: (kwc) I really want to nix this feature. It creates
+        # extra special codepaths and isn't as important now that the
+        # underlay feature exists.
         
         # Load ros_home/rosdep.yaml, if present.  It will be used to
         # override individual stack views.
@@ -328,7 +326,17 @@ class RosdepLookup(object):
                 data = loader.load_rosdep_yaml(f.read(), path)
             override_entry = RosdepDatabaseEntry(data, [], path)
 
-        return RosdepLookup(rosdep_db, loader, override_entry=override_entry)
+        # create our actual instance
+        lookup = RosdepLookup(rosdep_db, loader, override_entry=override_entry)
+
+        # load in the underlay
+        if use_underlay:
+            lookup._load_all_views(loader=underlay_loader)
+            # use dependencies to implement precedence
+            view_dependencies = underlay_loader.get_loadable_views()
+            rosdep_db.set_view_data(underlay_key, {}, view_dependencies, underlay_key)
+
+        return lookup
 
     def resolve_all(self, resources, installer_context):
         """
@@ -364,11 +372,6 @@ class RosdepLookup(object):
             except ResourceNotFound as e:
                 errors[resource_name] = e
 
-        # check for dependencies
-        for installer_key, val in resolutions.items(): #py3k
-            for d in dependencies:
-                self.install_rosdep(d, rdlp, default_yes, execute)
-
         # consolidate resolutions 
         for installer_key, val in resolutions.items(): #py3k
             installer = installer_context.get_installer(installer_key)
@@ -440,24 +443,25 @@ class RosdepLookup(object):
 
         return installer_key, resolution, dependencies
         
-    def _load_all_views(self):
+    def _load_all_views(self, loader):
         """
         Load all available view keys.  In general, this is equivalent
         to loading all stacks on the package path.  If
-        :exc:`InvalidRosdepData` errors occur while loading a view,
+        :exc:`InvalidData` errors occur while loading a view,
         they will be saved in the *errors* field.
 
+        :param loader: override self.loader
         :raises: :exc:`RosdepInternalError` 
         """
-        for resource_name in self.loader.get_loadable_views():
+        for resource_name in loader.get_loadable_views():
             try:
-                self._load_view_dependencies(resource_name)
+                self._load_view_dependencies(resource_name, loader)
             except ResourceNotFound as e:
                 self.errors.append(e)
-            except InvalidRosdepData as e:
+            except InvalidData as e:
                 self.errors.append(e)
         
-    def _load_view_dependencies(self, view_key):
+    def _load_view_dependencies(self, view_key, loader):
         """
         Initialize internal :exc:`RosdepDatabase` on demand.  Not
         thread-safe.
@@ -465,7 +469,7 @@ class RosdepLookup(object):
         :param view_key: name of view to load dependencies for.
         
         :raises: :exc:`rospkg.ResourceNotFound` If view cannot be located
-        :raises: :exc:`InvalidRosdepData` if view's data is invaid
+        :raises: :exc:`InvalidData` if view's data is invaid
         :raises: :exc:`RosdepInternalError`
         """
         rd_debug("_load_view_dependencies[%s]"%(view_key))
@@ -473,14 +477,14 @@ class RosdepLookup(object):
         if db.is_loaded(view_key):
             return
         try:
-            self.loader.load_view(view_key, db, verbose=self.verbose)
+            loader.load_view(view_key, db, verbose=self.verbose)
             entry = db.get_view_data(view_key)
             rd_debug("_load_view_dependencies[%s]: %s"%(view_key, entry.view_dependencies))            
             for d in entry.view_dependencies:
-                self._load_view_dependencies(d)
-        except InvalidRosdepData:
+                self._load_view_dependencies(d, loader)
+        except InvalidData:
             # mark view as loaded: as we are caching, the valid
-            # behavior is to not attempt loading this stack ever
+            # behavior is to not attempt loading this view ever
             # again.
             db.mark_loaded(view_key)
             # re-raise
@@ -488,9 +492,11 @@ class RosdepLookup(object):
         except KeyError as e:
             raise RosdepInternalError(e)
     
-    def create_rosdep_view(self, view_name, view_keys):
+    def create_rosdep_view(self, view_name, view_keys, verbose=False):
         """
-        :raises: :exc:`RosdepConflict` If view cannot be created due to conflicting definitions.
+        :param view_name: name of view to create
+        :param view_keys: order list of view names to merge, first one wins
+        :param verbose: print debugging output
         """
         # Create view and initialize with dbs from all of the
         # dependencies. 
@@ -499,14 +505,16 @@ class RosdepLookup(object):
         db = self.rosdep_db
         for view_key in view_keys:
             db_entry = db.get_view_data(view_key)
-            view.merge(db_entry)
-
+            view.merge(db_entry, verbose=verbose)
+        if verbose:
+            print("Merged views:\n"+"\n".join([" * %s"%view_key for view_key in view_keys]))
+            
         # ~/.ros/rosdep.yaml has precedence
         if self.override_entry is not None:
-            view.merge(self.override_entry, override=True)
+            view.merge(self.override_entry, override=True, verbose=verbose)
         return view
     
-    def get_rosdep_view_for_resource(self, resource_name):
+    def get_rosdep_view_for_resource(self, resource_name, verbose=False):
         """
         Get a :class:`RosdepView` for a specific ROS resource *resource_name*.
         Views can be queries to resolve rosdep keys to
@@ -528,9 +536,9 @@ class RosdepLookup(object):
             #NOTE: this may not be the right behavior and this happens
             #for packages that are not in a stack.
             return None
-        return self.get_rosdep_view(view_key)
+        return self.get_rosdep_view(view_key, verbose=verbose)
         
-    def get_rosdep_view(self, view_key):
+    def get_rosdep_view(self, view_key, verbose=False):
         """
         Get a :class:`RosdepView` associated with *view_key*.  Views
         can be queries to resolve rosdep keys to definitions.
@@ -546,7 +554,7 @@ class RosdepLookup(object):
             return self._view_cache[view_key]
 
         # lazy-init
-        self._load_view_dependencies(view_key)
+        self._load_view_dependencies(view_key, self.loader)
 
         # use dependencies to create view
         try:
@@ -555,7 +563,8 @@ class RosdepLookup(object):
             # convert to ResourceNotFound.  This should be decoupled
             # in the future
             raise ResourceNotFound(str(e.args[0]))
-        view = self.create_rosdep_view(view_key, [view_key] + dependencies)
+        # load views in order
+        view = self.create_rosdep_view(view_key, dependencies + [view_key], verbose=verbose)
         self._view_cache[view_key] = view
         return view
 
@@ -566,7 +575,7 @@ class RosdepLookup(object):
         in the configuration will be loaded into memory.
 
         Error state from single-stack failures
-        (e.g. :exc:`InvalidRosdepData`, :exc:`ResourceNotFound`) are
+        (e.g. :exc:`InvalidData`, :exc:`ResourceNotFound`) are
         not propagated.  Caller must check
         :meth:`RosdepLookup.get_errors` to check for single-stack
         error state.  Error state does not reset -- it accumulates.
@@ -577,7 +586,7 @@ class RosdepLookup(object):
         :raises: :exc:`RosdepInternalError` 
         """
         #TODOXXX: change this to return errors object so that caller cannot ignore
-        self._load_all_views()
+        self._load_all_views(self.loader)
         db = self.rosdep_db
         retval = []
         for view_name in db.get_view_names():
